import aquila_borg as borg
import configparser
import os

# Output stream management
cons = borg.console()
myprint = lambda x: cons.print_std(x) if type(x) == str else cons.print_std(repr(x))


def get_cosmopar(ini_file):
    """
    Extract cosmological parameters from an ini file
    
    Args:
        :ini_file (str): Path to the ini file
    
    Returns:
        :cpar (borg.cosmo.CosmologicalParameters): Cosmological parameters
    """
    
    config = configparser.ConfigParser()
    config.read(ini_file)
        
    cpar = borg.cosmo.CosmologicalParameters()
    cpar.default()
    cpar.fnl = float(config['cosmology']['fnl'])
    cpar.omega_k = float(config['cosmology']['omega_k'])
    cpar.omega_m = float(config['cosmology']['omega_m'])
    cpar.omega_b = float(config['cosmology']['omega_b'])
    cpar.omega_q = float(config['cosmology']['omega_q'])
    cpar.h = float(config['cosmology']['h100'])
    cpar.sigma8 = float(config['cosmology']['sigma8'])
    cpar.n_s = float(config['cosmology']['n_s'])
    cpar.w = float(config['cosmology']['w'])
    cpar.wprime = float(config['cosmology']['wprime'])
    
    return cpar


def get_action():
    """
    Find mode which BORG is currently being run in
    
    Returns:
        :last_line (str): The mode in which BORG is being run
    """
    
    # Find the last line of the file
    with open('trace_file.txt', 'rb') as file:
        file.seek(-2, os.SEEK_END)  # Go to the second last byte in the file.
        last_line = ''
        while len(last_line) == 0:
            while file.read(1) != b'\n':  # Keep moving back until you find a newline.
                file.seek(-2, os.SEEK_CUR)
            last_line = file.readline().decode().strip('\n')
            
    # Check that the this is the line was want and find action
    cmd = "hades_python"
    assert cmd in last_line
    idx = last_line.index(cmd)
    last_line = last_line[idx+len(cmd):]
    while last_line[0] == ' ':
        last_line = last_line[1:]
    idx = last_line.index(' ')
    last_line = last_line[:idx].upper()
    myprint(f'Running BORG mode: {last_line}')
    
    return last_line